# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/SymbolicExpression.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/SymbolicExpression.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_options=_b('\n\032com.grammatech.gtirb.proto'),
  serialized_pb=_b('\n$gtirb/proto/SymbolicExpression.proto\x12\x0bgtirb.proto\"4\n\rSymStackConst\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x13\n\x0bsymbol_uuid\x18\x02 \x01(\x0c\"3\n\x0cSymAddrConst\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x13\n\x0bsymbol_uuid\x18\x02 \x01(\x0c\"X\n\x0bSymAddrAddr\x12\r\n\x05scale\x18\x01 \x01(\x03\x12\x0e\n\x06offset\x18\x02 \x01(\x03\x12\x14\n\x0csymbol1_uuid\x18\x03 \x01(\x0c\x12\x14\n\x0csymbol2_uuid\x18\x04 \x01(\x0c\"\xc4\x01\n\x12SymbolicExpression\x12/\n\naddr_const\x18\x02 \x01(\x0b\x32\x19.gtirb.proto.SymAddrConstH\x00\x12-\n\taddr_addr\x18\x03 \x01(\x0b\x32\x18.gtirb.proto.SymAddrAddrH\x00\x12\x32\n\x0f\x61ttribute_flags\x18\x04 \x03(\x0e\x32\x19.gtirb.proto.SymAttributeB\x07\n\x05valueJ\x04\x08\x01\x10\x02R\x0bstack_const*\xec\x05\n\x0cSymAttribute\x12\x07\n\x03GOT\x10\x00\x12\t\n\x05GOTPC\x10\x01\x12\n\n\x06GOTOFF\x10\x02\x12\n\n\x06GOTREL\x10\x03\x12\x07\n\x03PLT\x10\x04\x12\n\n\x06PLTOFF\x10\x05\x12\t\n\x05PCREL\x10\x06\x12\n\n\x06SECREL\x10\x07\x12\x07\n\x03TLS\x10\x08\x12\t\n\x05TLSGD\x10\t\x12\t\n\x05TLSLD\x10\n\x12\n\n\x06TLSLDM\x10\x0b\x12\x0b\n\x07TLSCALL\x10\x0c\x12\x0b\n\x07TLSDESC\x10\r\x12\t\n\x05TPREL\x10\x0e\x12\t\n\x05TPOFF\x10\x0f\x12\n\n\x06\x44TPREL\x10\x10\x12\n\n\x06\x44TPOFF\x10\x11\x12\n\n\x06NTPOFF\x10\x12\x12\n\n\x06\x44TPMOD\x10\x13\x12\x08\n\x04PAGE\x10\x14\x12\x0b\n\x07PAGEOFF\x10\x15\x12\x08\n\x04\x43\x41LL\x10\x16\x12\x06\n\x02LO\x10\x17\x12\x06\n\x02HI\x10\x18\x12\n\n\x06HIGHER\x10\x19\x12\x0b\n\x07HIGHEST\x10\x1a\x12\x0e\n\tGOTNTPOFF\x10\xe8\x07\x12\x0e\n\tINDNTPOFF\x10\xe9\x07\x12\x07\n\x02G0\x10\xd1\x0f\x12\x07\n\x02G1\x10\xd2\x0f\x12\x07\n\x02G2\x10\xd3\x0f\x12\x07\n\x02G3\x10\xd4\x0f\x12\x0c\n\x07UPPER16\x10\xd5\x0f\x12\x0c\n\x07LOWER16\x10\xd6\x0f\x12\t\n\x04LO12\x10\xd7\x0f\x12\t\n\x04LO15\x10\xd8\x0f\x12\t\n\x04LO14\x10\xd9\x0f\x12\t\n\x04HI12\x10\xda\x0f\x12\t\n\x04HI21\x10\xdb\x0f\x12\x06\n\x01S\x10\xdc\x0f\x12\x07\n\x02PG\x10\xdd\x0f\x12\x07\n\x02NC\x10\xde\x0f\x12\x08\n\x03\x41\x42S\x10\xdf\x0f\x12\t\n\x04PREL\x10\xe0\x0f\x12\x0b\n\x06PREL31\x10\xe1\x0f\x12\x0c\n\x07TARGET1\x10\xe2\x0f\x12\x0c\n\x07TARGET2\x10\xe3\x0f\x12\n\n\x05SBREL\x10\xe4\x0f\x12\x0b\n\x06TLSLDO\x10\xe5\x0f\x12\t\n\x04HI16\x10\xb8\x17\x12\t\n\x04LO16\x10\xb9\x17\x12\n\n\x05GPREL\x10\xba\x17\x12\t\n\x04\x44ISP\x10\xbb\x17\x12\t\n\x04OFST\x10\xbc\x17\x12\x06\n\x01H\x10\xa0\x1f\x12\x06\n\x01L\x10\xa1\x1f\x12\x07\n\x02HA\x10\xa2\x1f\x12\t\n\x04HIGH\x10\xa3\x1f\x12\n\n\x05HIGHA\x10\xa4\x1f\x12\x0c\n\x07HIGHERA\x10\xa5\x1f\x12\r\n\x08HIGHESTA\x10\xa6\x1f\x12\x0c\n\x07TOCBASE\x10\xa7\x1f\x12\x08\n\x03TOC\x10\xa8\x1f\x12\n\n\x05NOTOC\x10\xa9\x1f\x42\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
)

_SYMATTRIBUTE = _descriptor.EnumDescriptor(
  name='SymAttribute',
  full_name='gtirb.proto.SymAttribute',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='GOT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GOTPC', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GOTOFF', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GOTREL', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PLT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PLTOFF', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PCREL', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SECREL', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLS', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSGD', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSLD', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSLDM', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSCALL', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSDESC', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TPREL', index=14, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TPOFF', index=15, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DTPREL', index=16, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DTPOFF', index=17, number=17,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NTPOFF', index=18, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DTPMOD', index=19, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PAGE', index=20, number=20,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PAGEOFF', index=21, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CALL', index=22, number=22,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LO', index=23, number=23,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HI', index=24, number=24,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHER', index=25, number=25,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHEST', index=26, number=26,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GOTNTPOFF', index=27, number=1000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INDNTPOFF', index=28, number=1001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='G0', index=29, number=2001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='G1', index=30, number=2002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='G2', index=31, number=2003,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='G3', index=32, number=2004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPPER16', index=33, number=2005,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOWER16', index=34, number=2006,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LO12', index=35, number=2007,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LO15', index=36, number=2008,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LO14', index=37, number=2009,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HI12', index=38, number=2010,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HI21', index=39, number=2011,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='S', index=40, number=2012,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PG', index=41, number=2013,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NC', index=42, number=2014,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ABS', index=43, number=2015,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PREL', index=44, number=2016,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PREL31', index=45, number=2017,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TARGET1', index=46, number=2018,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TARGET2', index=47, number=2019,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SBREL', index=48, number=2020,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TLSLDO', index=49, number=2021,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HI16', index=50, number=3000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LO16', index=51, number=3001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GPREL', index=52, number=3002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DISP', index=53, number=3003,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFST', index=54, number=3004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='H', index=55, number=4000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='L', index=56, number=4001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HA', index=57, number=4002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGH', index=58, number=4003,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHA', index=59, number=4004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHERA', index=60, number=4005,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHESTA', index=61, number=4006,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TOCBASE', index=62, number=4007,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TOC', index=63, number=4008,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOTOC', index=64, number=4009,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=450,
  serialized_end=1198,
)
_sym_db.RegisterEnumDescriptor(_SYMATTRIBUTE)

SymAttribute = enum_type_wrapper.EnumTypeWrapper(_SYMATTRIBUTE)
GOT = 0
GOTPC = 1
GOTOFF = 2
GOTREL = 3
PLT = 4
PLTOFF = 5
PCREL = 6
SECREL = 7
TLS = 8
TLSGD = 9
TLSLD = 10
TLSLDM = 11
TLSCALL = 12
TLSDESC = 13
TPREL = 14
TPOFF = 15
DTPREL = 16
DTPOFF = 17
NTPOFF = 18
DTPMOD = 19
PAGE = 20
PAGEOFF = 21
CALL = 22
LO = 23
HI = 24
HIGHER = 25
HIGHEST = 26
GOTNTPOFF = 1000
INDNTPOFF = 1001
G0 = 2001
G1 = 2002
G2 = 2003
G3 = 2004
UPPER16 = 2005
LOWER16 = 2006
LO12 = 2007
LO15 = 2008
LO14 = 2009
HI12 = 2010
HI21 = 2011
S = 2012
PG = 2013
NC = 2014
ABS = 2015
PREL = 2016
PREL31 = 2017
TARGET1 = 2018
TARGET2 = 2019
SBREL = 2020
TLSLDO = 2021
HI16 = 3000
LO16 = 3001
GPREL = 3002
DISP = 3003
OFST = 3004
H = 4000
L = 4001
HA = 4002
HIGH = 4003
HIGHA = 4004
HIGHERA = 4005
HIGHESTA = 4006
TOCBASE = 4007
TOC = 4008
NOTOC = 4009



_SYMSTACKCONST = _descriptor.Descriptor(
  name='SymStackConst',
  full_name='gtirb.proto.SymStackConst',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymStackConst.offset', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol_uuid', full_name='gtirb.proto.SymStackConst.symbol_uuid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=105,
)


_SYMADDRCONST = _descriptor.Descriptor(
  name='SymAddrConst',
  full_name='gtirb.proto.SymAddrConst',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymAddrConst.offset', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol_uuid', full_name='gtirb.proto.SymAddrConst.symbol_uuid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=158,
)


_SYMADDRADDR = _descriptor.Descriptor(
  name='SymAddrAddr',
  full_name='gtirb.proto.SymAddrAddr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scale', full_name='gtirb.proto.SymAddrAddr.scale', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymAddrAddr.offset', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol1_uuid', full_name='gtirb.proto.SymAddrAddr.symbol1_uuid', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol2_uuid', full_name='gtirb.proto.SymAddrAddr.symbol2_uuid', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=160,
  serialized_end=248,
)


_SYMBOLICEXPRESSION = _descriptor.Descriptor(
  name='SymbolicExpression',
  full_name='gtirb.proto.SymbolicExpression',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='addr_const', full_name='gtirb.proto.SymbolicExpression.addr_const', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='addr_addr', full_name='gtirb.proto.SymbolicExpression.addr_addr', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_flags', full_name='gtirb.proto.SymbolicExpression.attribute_flags', index=2,
      number=4, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='gtirb.proto.SymbolicExpression.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=251,
  serialized_end=447,
)

_SYMBOLICEXPRESSION.fields_by_name['addr_const'].message_type = _SYMADDRCONST
_SYMBOLICEXPRESSION.fields_by_name['addr_addr'].message_type = _SYMADDRADDR
_SYMBOLICEXPRESSION.fields_by_name['attribute_flags'].enum_type = _SYMATTRIBUTE
_SYMBOLICEXPRESSION.oneofs_by_name['value'].fields.append(
  _SYMBOLICEXPRESSION.fields_by_name['addr_const'])
_SYMBOLICEXPRESSION.fields_by_name['addr_const'].containing_oneof = _SYMBOLICEXPRESSION.oneofs_by_name['value']
_SYMBOLICEXPRESSION.oneofs_by_name['value'].fields.append(
  _SYMBOLICEXPRESSION.fields_by_name['addr_addr'])
_SYMBOLICEXPRESSION.fields_by_name['addr_addr'].containing_oneof = _SYMBOLICEXPRESSION.oneofs_by_name['value']
DESCRIPTOR.message_types_by_name['SymStackConst'] = _SYMSTACKCONST
DESCRIPTOR.message_types_by_name['SymAddrConst'] = _SYMADDRCONST
DESCRIPTOR.message_types_by_name['SymAddrAddr'] = _SYMADDRADDR
DESCRIPTOR.message_types_by_name['SymbolicExpression'] = _SYMBOLICEXPRESSION
DESCRIPTOR.enum_types_by_name['SymAttribute'] = _SYMATTRIBUTE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SymStackConst = _reflection.GeneratedProtocolMessageType('SymStackConst', (_message.Message,), dict(
  DESCRIPTOR = _SYMSTACKCONST,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymStackConst)
  ))
_sym_db.RegisterMessage(SymStackConst)

SymAddrConst = _reflection.GeneratedProtocolMessageType('SymAddrConst', (_message.Message,), dict(
  DESCRIPTOR = _SYMADDRCONST,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymAddrConst)
  ))
_sym_db.RegisterMessage(SymAddrConst)

SymAddrAddr = _reflection.GeneratedProtocolMessageType('SymAddrAddr', (_message.Message,), dict(
  DESCRIPTOR = _SYMADDRADDR,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymAddrAddr)
  ))
_sym_db.RegisterMessage(SymAddrAddr)

SymbolicExpression = _reflection.GeneratedProtocolMessageType('SymbolicExpression', (_message.Message,), dict(
  DESCRIPTOR = _SYMBOLICEXPRESSION,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymbolicExpression)
  ))
_sym_db.RegisterMessage(SymbolicExpression)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
